EXPORTS
	?DecodeDERL@TASN1DecPKCS5@@SAPAVCPBEncryptParms@@ABVTDesC8@@@Z @ 1 NONAME ; public: static class CPBEncryptParms * __cdecl TASN1DecPKCS5::DecodeDERL(class TDesC8 const &)
	?DecodeDERL@TASN1DecPKCS8@@SAPAVCDecPKCS8Data@@ABVTDesC8@@@Z @ 2 NONAME ; public: static class CDecPKCS8Data * __cdecl TASN1DecPKCS8::DecodeDERL(class TDesC8 const &)
	?DecodeEncryptedDERL@TASN1DecPKCS8@@SAPAVCDecPKCS8Data@@ABVTDesC8@@0@Z @ 3 NONAME ; public: static class CDecPKCS8Data * __cdecl TASN1DecPKCS8::DecodeEncryptedDERL(class TDesC8 const &,class TDesC8 const &)
	?EncodeDERL@TASN1EncPKCS5@@SAPAVCASN1EncSequence@@ABVCPBEncryptParms@@@Z @ 4 NONAME ; public: static class CASN1EncSequence * __cdecl TASN1EncPKCS5::EncodeDERL(class CPBEncryptParms const &)
	?EncodeEncryptedL@TASN1EncPKCS8@@SAPAVCASN1EncSequence@@ABVCDSAPrivateKey@@AAVCPBEncryptor@@AAVCPBEncryptParms@@ABVTDesC8@@@Z @ 5 NONAME ; public: static class CASN1EncSequence * __cdecl TASN1EncPKCS8::EncodeEncryptedL(class CDSAPrivateKey const &,class CPBEncryptor &,class CPBEncryptParms &,class TDesC8 &)
	?EncodeEncryptedL@TASN1EncPKCS8@@SAPAVCASN1EncSequence@@ABVCRSAPrivateKeyCRT@@ABVCRSAPublicKey@@AAVCPBEncryptor@@AAVCPBEncryptParms@@ABVTDesC8@@@Z @ 6 NONAME ; public: static class CASN1EncSequence * __cdecl TASN1EncPKCS8::EncodeEncryptedL(class CRSAPrivateKeyCRT const &,class CRSAPublicKey const &,class CPBEncryptor &,class CPBEncryptParms &,class TDesC8 &)
	?EncodeL@TASN1EncPKCS8@@SAPAVCASN1EncSequence@@ABVCDSAPrivateKey@@ABVTDesC8@@@Z @ 7 NONAME ; public: static class CASN1EncSequence * __cdecl TASN1EncPKCS8::EncodeL(class CDSAPrivateKey const &, class TDesC8 &)
	?EncodeL@TASN1EncPKCS8@@SAPAVCASN1EncSequence@@ABVCRSAPrivateKeyCRT@@ABVCRSAPublicKey@@ABVTDesC8@@@Z @ 8 NONAME ; public: static class CASN1EncSequence * __cdecl TASN1EncPKCS8::EncodeL(class CRSAPrivateKeyCRT const &,class CRSAPublicKey const &,class TDesC8 &)
	?IsEncryptedPKCS8Data@TASN1DecPKCS8@@SAHABVTDesC8@@@Z @ 9 NONAME ; int TASN1DecPKCS8::IsEncryptedPKCS8Data(class TDesC8 const &)
	?IsPKCS8Data@TASN1DecPKCS8@@SAHABVTDesC8@@@Z @ 10 NONAME ; int TASN1DecPKCS8::IsPKCS8Data(class TDesC8 const &)

